@@ -10,6 +10,7 @@ import queue
 import socket
 import threading
 import uuid
+import warnings
 from collections import namedtuple
 
 
@@ -23,6 +24,7 @@ class Client:
         self._response = queue.Queue(maxsize=1)
         self._sid = 0
         self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        self._run = False
 
     def get(self):
         """returns list of messages received since get was last called"""
@@ -30,62 +32,69 @@ class Client:
 
     def publish(self, *, subject, payload=""):
         """publish payload on subject"""
-        self._sock.send(
-            f"PUB {subject} {len(payload)}\r\n{payload}\r\n".encode("utf-8")
-        )
+        self._send(f"PUB {subject} {len(payload)}\r\n{payload}")
 
     def subscribe(self, *, subject):
         """subscribe to subject"""
         self._sid += 1
-        self._sock.send(f"SUB {subject} {self._sid}\r\n".encode("utf-8"))
+        self._send(f"SUB {subject} {self._sid}")
 
     def request(self, *, subject, payload=""):
         """request subject for a response to payload"""
         inbox = f"INBOX.{uuid.uuid4().hex}"
         self._sid += 1
-        self._sock.send(f"SUB {inbox} {self._sid}\r\n".encode("utf-8"))
-        self._sock.send(f"UNSUB {self._sid} 1\r\n".encode("utf-8"))
-        self._sock.send(
-            f"PUB {subject} {inbox} {len(payload)}\r\n{payload}\r\n".encode("utf-8")
-        )
+        self._send(f"SUB {inbox} {self._sid}")
+        self._send(f"UNSUB {self._sid} 1")
+        self._send(f"PUB {subject} {inbox} {len(payload)}\r\n{payload}")
         return self._response.get()
 
-    def _connect(self):
-        self._sock.connect((self.host, self.port))
-        threading.Thread(target=self._thread, daemon=True).start()
-
-    def _disconnect(self):
-        self._sock.shutdown(socket.SHUT_RDWR)
-        self._sock.close()
-
     def __enter__(self):
-        self._connect()
+        try:
+            self._sock.connect((self.host, self.port))
+        except ConnectionRefusedError as e:
+            raise ConnectionRefusedError(
+                f"can't connect to {self.host}:{self.port}"
+            ) from e
+        self._run = True
+        threading.Thread(target=self._thread, daemon=True).start()
         return self
 
     def __exit__(self, exception_type, exception_value, exception_traceback):
-        self._disconnect()
+        self._run = False
+        try:
+            self._sock.shutdown(socket.SHUT_RDWR)
+        except OSError:
+            pass
+        self._sock.close()
+
+    def _send(self, string):
+        try:
+            self._sock.sendall(f"{string}\r\n".encode("utf-8"))
+        except BrokenPipeError as e:
+            raise BrokenPipeError("send failed") from e
 
     def _thread(self):
         # https://docs.nats.io/nats-protocol/nats-protocol
-        while True:
-            received = self._sock.recv(4096)
+        while self._run:
+            try:
+                received = self._sock.recv(4096)
+            except ConnectionResetError as e:
+                raise ConnectionResetError("recv failed") from e
             current, eom, next = received.partition(b"\r\n")
             self._buffer += current
             # End Of Message
             if eom:
                 if self._buffer == b"PING":
-                    self._sock.send(b"PONG\r\n")
+                    self._send("PONG")
                 elif self._buffer == b"PONG":
                     pass
                 elif self._buffer == b"+OK":
                     pass
                 elif self._buffer.startswith(b"-ERR"):
-                    print(self._buffer)
+                    raise warnings.warn(self._buffer.decode("utf-8"))
                 elif self._buffer.startswith(b"INFO"):
-                    self._sock.send(
-                        f'CONNECT {{"name": "{self.name}", "verbose": false}}\r\n'.encode(
-                            "utf-8"
-                        )
+                    self._send(
+                        f'CONNECT {{"name": "{self.name}", "verbose": false}}'
                     )
                 elif self._buffer.startswith(b"MSG"):
                     # MSG ...  \r\n[payload]\r\n
@@ -123,6 +132,7 @@ Message = namedtuple("Message", "subject payload")
 Request = namedtuple("Request", "subject inbox payload")
 Response = namedtuple("Response", "payload")
 
+
 if __name__ == "__main__":
     import datetime as dt
     import time
